package version

// Version indicates which version of the binary is running.
var Version = "master"

// GitCommitSHA indicates which git shorthash the binary was built off of
var GitCommitSHA string
