// Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//	http://aws.amazon.com/apache2.0/
//
// or in the "license" file accompanying this file. This file is distributed
// on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
// express or implied. See the License for the specific language governing
// permissions and limitations under the License.

package cache

type nullCredentialsCache struct{}

func NewNullCredentialsCache() CredentialsCache {
	return &nullCredentialsCache{}
}

func (n *nullCredentialsCache) Get(_ string) *AuthEntry {
	return nil
}

func (n *nullCredentialsCache) GetPublic() *AuthEntry {
	return nil
}

func (n *nullCredentialsCache) Set(_ string, _ *AuthEntry) {
}

func (n *nullCredentialsCache) List() []*AuthEntry {
	return []*AuthEntry{}
}

func (n *nullCredentialsCache) Clear() {
}
